from django import test

from ievv_opensource.ievv_customsql import customsql_registry


class TestRegistry(test.TestCase):
    def test_add(self):
        class MockCustomSql(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        mockregistry.add('myapp', MockCustomSql)
        self.assertIn(MockCustomSql, mockregistry._customsql_classes)
        self.assertIn('myapp', mockregistry._customsql_classes_by_appname_map)
        self.assertIn(MockCustomSql, mockregistry._customsql_classes_by_appname_map['myapp'])

    def test_add_multiple_to_same_appname(self):
        class MockCustomSql1(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql2(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        mockregistry.add('myapp', MockCustomSql1)
        mockregistry.add('myapp', MockCustomSql2)
        self.assertIn(MockCustomSql1, mockregistry._customsql_classes)
        self.assertIn(MockCustomSql2, mockregistry._customsql_classes)
        self.assertIn('myapp', mockregistry._customsql_classes_by_appname_map)
        self.assertIn(MockCustomSql1, mockregistry._customsql_classes_by_appname_map['myapp'])
        self.assertIn(MockCustomSql2, mockregistry._customsql_classes_by_appname_map['myapp'])

    def test_remove(self):
        class MockCustomSql(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        mockregistry.add('myapp', MockCustomSql)

        mockregistry.remove('myapp', MockCustomSql)
        self.assertNotIn(MockCustomSql, mockregistry._customsql_classes)
        self.assertNotIn('myapp', mockregistry._customsql_classes_by_appname_map)

    def test_contains(self):
        class MockCustomSql(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        self.assertFalse(MockCustomSql in mockregistry)
        mockregistry.add('myapp', MockCustomSql)
        self.assertTrue(MockCustomSql in mockregistry)

    def test_remove_one_of_multiple_in_same_app(self):
        class MockCustomSql1(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql2(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        mockregistry.add('myapp', MockCustomSql1)
        mockregistry.add('myapp', MockCustomSql2)

        mockregistry.remove('myapp', MockCustomSql2)
        self.assertNotIn(MockCustomSql2, mockregistry._customsql_classes)
        self.assertNotIn(MockCustomSql2, mockregistry._customsql_classes_by_appname_map['myapp'])
        self.assertIn(MockCustomSql1, mockregistry._customsql_classes)
        self.assertIn('myapp', mockregistry._customsql_classes_by_appname_map)

    def test_iter(self):
        class MockCustomSql1(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql2(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql3(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        mockregistry.add('my_first_app', MockCustomSql1)
        mockregistry.add('my_first_app', MockCustomSql2)
        mockregistry.add('my_second_app', MockCustomSql3)

        iter_list = list(mockregistry)
        self.assertTrue(isinstance(iter_list[0], MockCustomSql1))
        self.assertTrue(isinstance(iter_list[1], MockCustomSql2))
        self.assertTrue(isinstance(iter_list[2], MockCustomSql3))

    def test_iter_appnames(self):
        class MockCustomSql1(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql2(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql3(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        mockregistry.add('my_first_app', MockCustomSql1)
        mockregistry.add('my_first_app', MockCustomSql2)
        mockregistry.add('my_second_app', MockCustomSql3)

        self.assertEqual(['my_first_app', 'my_second_app'],
                         list(mockregistry.iter_appnames()))

    def test_iter_customsql_in_appname(self):
        class MockCustomSql1(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql2(customsql_registry.AbstractCustomSql):
            pass

        class MockCustomSql3(customsql_registry.AbstractCustomSql):
            pass

        mockregistry = customsql_registry.MockableRegistry()
        mockregistry.add('my_first_app', MockCustomSql1)
        mockregistry.add('my_first_app', MockCustomSql2)
        mockregistry.add('my_second_app', MockCustomSql3)

        iter_list = list(mockregistry.iter_customsql_in_app('my_first_app'))
        self.assertEqual(2, len(iter_list))
        self.assertTrue(isinstance(iter_list[0], MockCustomSql1))
        self.assertTrue(isinstance(iter_list[1], MockCustomSql2))


class TestAbstractCustomSqlCreateDropStatementsFromSqlCode(test.TestCase):
    def test_drop_trigger_drop_statements_from_sql_code(self):
        result = customsql_registry.AbstractCustomSql().make_drop_trigger_statements_from_sql_code(
            """
            CREATE TRIGGER my_fancy_trigger
                BEFORE INSERT OR UPDATE ON my_table
                FOR EACH ROW
                    EXECUTE PROCEDURE my_fancy_function();
            create trigger my_simple_trigger
                after insert on my_table
                for each row
                    execute procedure my_simple_function();
            """)
        self.assertEqual(result, [
            'DROP TRIGGER IF EXISTS my_fancy_trigger ON my_table',
            'DROP TRIGGER IF EXISTS my_simple_trigger ON my_table',
        ])

    def test_make_drop_function_statements_from_sql_code(self):
        result = customsql_registry.AbstractCustomSql().make_drop_function_statements_from_sql_code(
            """
                create function mysimple_function()
                returns void as $$
                begin
                end;
                $$ language plpgsql;

                create function invalidfunction

                CREATE OR REPLACE FUNCTION myfancy_function(
                    firsarg bigint,
                    secondarg text
                )
                RETURNS text AS $$
                BEGIN
                END;
                $$ LANGUAGE plpgsql;
            """)
        self.assertEqual(result, [
            'DROP FUNCTION IF EXISTS mysimple_function () RESTRICT',
            'DROP FUNCTION IF EXISTS myfancy_function (firsarg bigint, secondarg text) RESTRICT',
        ])

    def test_make_drop_index_statements_from_sql_code(self):
        result = customsql_registry.AbstractCustomSql().make_drop_index_statements_from_sql_code(
            """
            CREATE INDEX my_search_index
                ON ievv_pageframework_pagecontent
                USING GIN (search_vector);

            create index my_other_search_index
                on ievv_pageframework_pagecontent
                using gin (search_vector);
            """)
        self.assertEqual(result, [
            'DROP INDEX IF EXISTS my_search_index RESTRICT',
            'DROP INDEX IF EXISTS my_other_search_index RESTRICT',
        ])

    def test_make_drop_type_statements_from_sql_code(self):
        result = customsql_registry.AbstractCustomSql().make_drop_type_statements_from_sql_code(
            """
            create type my_simple_type as (size bigint);

            CREATE TYPE my_fancy_type AS (
                firstarg      bigint,
                secondarg     bigint
                thirdarg      text
            );
            """)
        self.assertEqual(result, [
            'DROP TYPE IF EXISTS my_simple_type RESTRICT',
            'DROP TYPE IF EXISTS my_fancy_type RESTRICT',
        ])

    def test_make_drop_statements_from_sql_code(self):
        result = customsql_registry.AbstractCustomSql().make_drop_statements_from_sql_code(
            """
            create type my_simple_type as (size bigint);
            
            create index my_search_index
                on ievv_pageframework_pagecontent
                using gin (search_vector);
                
            create function mysimple_function()
            returns void as $$
            begin
            end;
            $$ language plpgsql;

            create trigger my_simple_trigger
            after insert on my_table
            for each row
                execute procedure my_simple_function();
            """)
        self.assertEqual(result, [
            'DROP INDEX IF EXISTS my_search_index RESTRICT',
            'DROP TYPE IF EXISTS my_simple_type RESTRICT',
            'DROP FUNCTION IF EXISTS mysimple_function () RESTRICT',
            'DROP TRIGGER IF EXISTS my_simple_trigger ON my_table',
        ])



import os
import shutil
import unittest

from django.core import management
from django.core.management import CommandError
from django.test import TestCase

from ievv_opensource.utils import virtualenvutils
from ievv_opensource.python2_compatibility import mock


@unittest.skip('This is both slow and hard to ensure works in different environments')
class TestDocs(TestCase):
    def __remove_documentation_build_directory(self):
        if os.path.exists(self.documentation_build_directory):
            shutil.rmtree(self.documentation_build_directory)

    def setUp(self):
        self.documentation_build_directory = 'test_documentation_build_directory'
        self.__remove_documentation_build_directory()
        virtualenvutils.add_virtualenv_bin_directory_to_path()

    def tearDown(self):
        self.__remove_documentation_build_directory()

    def test_fail_if_build_clean_or_open_is_not_specified(self):
        with self.settings(IEVVTASKS_DOCS_BUILD_DIRECTORY=self.documentation_build_directory):
            with self.assertRaises(CommandError):
                management.call_command('ievvtasks_docs')

    def test_build(self):
        indexhtml_path = os.path.join(self.documentation_build_directory,
                                      'index.html')
        self.assertFalse(os.path.exists(indexhtml_path))
        with self.settings(IEVVTASKS_DOCS_BUILD_DIRECTORY=self.documentation_build_directory):
            management.call_command('ievvtasks_docs', build=True)
        self.assertTrue(os.path.exists(self.documentation_build_directory))
        self.assertTrue(os.path.exists(indexhtml_path))

    def test_clean(self):
        os.makedirs(self.documentation_build_directory)
        self.assertTrue(os.path.exists(self.documentation_build_directory))
        with self.settings(IEVVTASKS_DOCS_BUILD_DIRECTORY=self.documentation_build_directory):
            management.call_command('ievvtasks_docs', clean=True)
        self.assertFalse(os.path.exists(self.documentation_build_directory))

    def test_opendocs(self):
        openfile_mock = mock.MagicMock()
        with self.settings(IEVVTASKS_DOCS_BUILD_DIRECTORY=self.documentation_build_directory):
            with mock.patch('ievv_opensource.ievvtasks_development.management.commands.'
                            'ievvtasks_docs.open_file_with_default_os_opener',
                            openfile_mock):
                management.call_command('ievvtasks_docs', open=True)
        openfile_mock.assert_called_once_with(os.path.join(
            self.documentation_build_directory, 'index.html'))

    def test_clean_and_open_not_build_fails(self):
        with self.settings(IEVVTASKS_DOCS_BUILD_DIRECTORY=self.documentation_build_directory):
            with self.assertRaises(CommandError):
                management.call_command('ievvtasks_docs', open=True, clean=True, build=False)



import os
import shutil
import unittest

from django.conf import settings

from django.core import management
from django.test import TestCase


@unittest.skip('This does not work as intended')
class TestMakemessages(TestCase):
    def __remove_locale_directory(self):
        if os.path.exists(self.locale_directory):
            shutil.rmtree(self.locale_directory)

    def setUp(self):
        self.locale_directory = settings.LOCALE_PATHS[0]
        self.locale_paths = [self.locale_directory]
        self.__remove_locale_directory()

    def tearDown(self):
        self.__remove_locale_directory()

    def test_locale_directory_created(self):
        self.assertFalse(os.path.exists(self.locale_directory))
        with self.settings(LOCALE_PATHS=self.locale_paths):
            management.call_command('ievvtasks_makemessages')
        self.assertTrue(os.path.exists(self.locale_directory))

    def test_po_file_created(self):
        po_file_path = os.path.join(self.locale_directory,
                                    'en', 'LC_MESSAGES', 'django.po')
        self.assertFalse(os.path.exists(po_file_path))
        with self.settings(IEVVTASKS_MAKEMESSAGES_LANGUAGE_CODES=['en']):
            management.call_command('ievvtasks_makemessages')
        self.assertTrue(os.path.exists(self.locale_directory))
        self.assertTrue(os.path.exists(po_file_path))



import os

from django.conf import settings
from django.core import management
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Compile translations.'

    def handle(self, *args, **options):
        current_directory = os.getcwd()
        for directory in getattr(settings, 'IEVVTASKS_MAKEMESSAGES_DIRECTORIES', [current_directory]):
            if isinstance(directory, dict):
                directory = directory['directory']
            os.chdir(directory)
            management.call_command('compilemessages')
            os.chdir(current_directory)



from django import test

from ievv_opensource.ievv_sms.phone_number_handler import NorwegianPhoneNumberHandler


class TestNorwegianPhoneNumberHandler(test.TestCase):
    def test_get_country_calling_code_starts_with_00(self):
        self.assertEqual(NorwegianPhoneNumberHandler('004712345678').get_country_calling_code(), 47)
        self.assertEqual(NorwegianPhoneNumberHandler('00 47 12 34 56 78').get_country_calling_code(), 47)
        self.assertIsNone(NorwegianPhoneNumberHandler('0047123456789').get_country_calling_code())
        self.assertIsNone(NorwegianPhoneNumberHandler('00471234567').get_country_calling_code())
        self.assertIsNone(NorwegianPhoneNumberHandler('004812345678').get_country_calling_code())

    def test_get_country_calling_code_starts_with_plus(self):
        self.assertEqual(NorwegianPhoneNumberHandler('+4712345678').get_country_calling_code(), 47)
        self.assertEqual(NorwegianPhoneNumberHandler('+ 47 12 34 56 78').get_country_calling_code(), 47)
        self.assertIsNone(NorwegianPhoneNumberHandler('+47123456789').get_country_calling_code())
        self.assertIsNone(NorwegianPhoneNumberHandler('+471234567').get_country_calling_code())
        self.assertIsNone(NorwegianPhoneNumberHandler('+4812345678').get_country_calling_code())

    def test_get_country_calling_code_no_code(self):
        self.assertEqual(NorwegianPhoneNumberHandler('12345678').get_country_calling_code(), 47)
        self.assertEqual(NorwegianPhoneNumberHandler('12 34 56 78').get_country_calling_code(), 47)
        self.assertIsNone(NorwegianPhoneNumberHandler('123456789').get_country_calling_code())
        self.assertIsNone(NorwegianPhoneNumberHandler('1234567').get_country_calling_code())

    def test_get_phone_number_without_country_code_starts_with_00(self):
        self.assertEqual(NorwegianPhoneNumberHandler('004712345678').get_phone_number_without_country_code(),
                         '12345678')
        self.assertEqual(NorwegianPhoneNumberHandler('00 47 12 34 56 78').get_phone_number_without_country_code(),
                         '12345678')
        self.assertIsNone(NorwegianPhoneNumberHandler('0047123456789').get_phone_number_without_country_code(),
                          '0047123456789')
        self.assertIsNone(NorwegianPhoneNumberHandler('00471234567').get_phone_number_without_country_code(),
                          '00471234567')
        self.assertIsNone(NorwegianPhoneNumberHandler('004812345678').get_phone_number_without_country_code(),
                          '004812345678')

    def test_get_phone_number_without_country_code_starts_with_plus(self):
        self.assertEqual(NorwegianPhoneNumberHandler('+4712345678').get_phone_number_without_country_code(),
                         '12345678')
        self.assertEqual(NorwegianPhoneNumberHandler('+ 47 12 34 56 78').get_phone_number_without_country_code(),
                         '12345678')
        self.assertIsNone(NorwegianPhoneNumberHandler('+47123456789').get_phone_number_without_country_code(),
                          '+47123456789')
        self.assertIsNone(NorwegianPhoneNumberHandler('+471234567').get_phone_number_without_country_code(),
                          '+471234567')
        self.assertIsNone(NorwegianPhoneNumberHandler('+4812345678').get_phone_number_without_country_code(),
                          '+4812345678')

    def test_get_phone_number_without_country_code_no_code(self):
        self.assertEqual(NorwegianPhoneNumberHandler('12345678').get_phone_number_without_country_code(),
                         '12345678')
        self.assertEqual(NorwegianPhoneNumberHandler('12 34 56 78').get_phone_number_without_country_code(),
                         '12345678')
        self.assertIsNone(NorwegianPhoneNumberHandler('123456789').get_phone_number_without_country_code(),
                          '123456789')
        self.assertIsNone(NorwegianPhoneNumberHandler('1234567').get_phone_number_without_country_code(),
                          '1234567')

    def test_can_receive_sms_starts_with_00(self):
        self.assertTrue(NorwegianPhoneNumberHandler('004792345678').can_receive_sms())
        self.assertTrue(NorwegianPhoneNumberHandler('00 47 92 34 56 78').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('004712345678').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('0047923456789').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('00479234567').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('004892345678').can_receive_sms())

    def test_can_receive_sms_starts_with_plus(self):
        self.assertTrue(NorwegianPhoneNumberHandler('+4792345678').can_receive_sms())
        self.assertTrue(NorwegianPhoneNumberHandler('+ 47 92 34 56 78').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('+4712345678').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('+47923456789').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('+479234567').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('+4892345678').can_receive_sms())

    def test_can_receive_sms_no_code(self):
        self.assertTrue(NorwegianPhoneNumberHandler('92345678').can_receive_sms())
        self.assertTrue(NorwegianPhoneNumberHandler('92 34 56 78').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('12345678').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('923456789').can_receive_sms())
        self.assertFalse(NorwegianPhoneNumberHandler('9234567').can_receive_sms())

    def test_cleaned_phone_number(self):
        self.assertEqual(NorwegianPhoneNumberHandler('004712345678').get_cleaned_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('+4712345678').get_cleaned_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('00 47 12 34 56 78').get_cleaned_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('+47 12 34 56 78').get_cleaned_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('12345678').get_cleaned_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('12 34 56 78').get_cleaned_phone_number(),
                         '+4712345678')
        self.assertIsNone(NorwegianPhoneNumberHandler('004812345678').get_cleaned_phone_number())
        self.assertIsNone(NorwegianPhoneNumberHandler('+4882345678').get_cleaned_phone_number())
        self.assertIsNone(NorwegianPhoneNumberHandler('0047123456789').get_cleaned_phone_number())
        self.assertIsNone(NorwegianPhoneNumberHandler('00471234567').get_cleaned_phone_number())

    def test_get_normalized_phone_number(self):
        self.assertEqual(NorwegianPhoneNumberHandler('004712345678').get_normalized_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('+4712345678').get_normalized_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('00 47 12 34 56 78').get_normalized_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('+47 12 34 56 78').get_normalized_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('12345678').get_normalized_phone_number(),
                         '+4712345678')
        self.assertEqual(NorwegianPhoneNumberHandler('12 34 56 78').get_normalized_phone_number(),
                         '+4712345678')

        self.assertEqual(NorwegianPhoneNumberHandler('004812345678').get_normalized_phone_number(), '004812345678')
        self.assertEqual(NorwegianPhoneNumberHandler('+4882345678').get_normalized_phone_number(), '+4882345678')
        self.assertEqual(NorwegianPhoneNumberHandler('0047123456789').get_normalized_phone_number(), '0047123456789')
        self.assertEqual(NorwegianPhoneNumberHandler('00471234567').get_normalized_phone_number(), '00471234567')

        self.assertEqual(NorwegianPhoneNumberHandler('0048 123 45 678').get_normalized_phone_number(), '004812345678')
        self.assertEqual(NorwegianPhoneNumberHandler('+488 2345 678').get_normalized_phone_number(), '+4882345678')
        self.assertEqual(NorwegianPhoneNumberHandler('00471 2  345  6789').get_normalized_phone_number(),
                         '0047123456789')
        self.assertEqual(NorwegianPhoneNumberHandler('004  712 34567').get_normalized_phone_number(), '00471234567')



from django.conf.urls import url
from django.utils.translation import ugettext_lazy

from . import views

urlpatterns = [
    url(r'^my/unnamed/untranslated_example$',
        views.unnamed_untranslated_exampleview),
    url(ugettext_lazy(r'^my/unnamed/translated_example$'),
        views.unnamed_translated_exampleview),
    url(r'^my/named/untranslated_example$',
        views.named_untranslated_exampleview,
        name="ievv_i18n_url_testapp_untranslated_exampleview"),
    url(ugettext_lazy(r'^my/named/translated_example$'),
        views.named_translated_exampleview,
        name="ievv_i18n_url_testapp_translated_exampleview"),
]



import re

from django.conf import settings
from django.core.exceptions import ValidationError


def is_valid_url(url):
    """
    Returns ``True`` if the provided URL matches the :setting:`IEVV_VALID_REDIRECT_URL_REGEX`
    regex.

    Args:
        url (str): An URL. Can just be a path too (E.g.: all of these work
            ``http://example.com``, ``/test``, ``http://example.com/test``.

    """
    allowed_redirect_url_regex = getattr(settings, 'IEVV_VALID_REDIRECT_URL_REGEX', r'^/.*$')
    return bool(re.match(allowed_redirect_url_regex, url))


def validate_url(url):
    """
    Validate the provided url against the regex in the :setting:`IEVV_VALID_REDIRECT_URL_REGEX`
    setting.

    Args:
        url (str): An URL. Can just be a path too (E.g.: all of these work
            ``http://example.com``, ``/test``, ``http://example.com/test``.

    Raises:
        django.core.exceptions.ValidationError: If the ``url`` is not valid.
    """
    if not is_valid_url(url):
        raise ValidationError('Invalid redirect URL: {}'.format(url))



from django import test
from django.core.exceptions import ValidationError
from django.test import override_settings

from ievv_opensource.utils import validate_redirect_url


class TestIsValidUrl(test.TestCase):
    def test_path_is_valid_by_default(self):
        self.assertTrue(validate_redirect_url.is_valid_url('/test/ing'))

    def test_domain_is_not_valid_by_default(self):
        self.assertFalse(validate_redirect_url.is_valid_url('http://example.com/test/ing'))

    @override_settings(IEVV_VALID_REDIRECT_URL_REGEX='^(https://example\.com/|/).*$')
    def test_custom_regex(self):
        self.assertTrue(validate_redirect_url.is_valid_url('https://example.com/test/ing'))
        self.assertTrue(validate_redirect_url.is_valid_url('https://example.com/'))
        self.assertTrue(validate_redirect_url.is_valid_url('/test/path'))
        self.assertTrue(validate_redirect_url.is_valid_url('/'))
        self.assertFalse(validate_redirect_url.is_valid_url('http://example.com/test/ing'))
        self.assertFalse(validate_redirect_url.is_valid_url('http://example.com/'))
        self.assertFalse(validate_redirect_url.is_valid_url('https://example2.com/'))
        self.assertFalse(validate_redirect_url.is_valid_url('https://example2.com/test/ing'))


class TestValidateUrl(test.TestCase):
    def test_path_is_valid_by_default(self):
        # No ValidationError
        validate_redirect_url.validate_url('/test/ing')

    def test_domain_is_not_valid_by_default(self):
        with self.assertRaises(ValidationError):
            validate_redirect_url.validate_url('http://example.com/test/ing')

    @override_settings(IEVV_VALID_REDIRECT_URL_REGEX=r'^/test$')
    def test_invalid_url(self):
        with self.assertRaises(ValidationError):
            validate_redirect_url.validate_url('/test2')

    @override_settings(IEVV_VALID_REDIRECT_URL_REGEX=r'^/test$')
    def test_valid_url(self):
        # No ValidationError
        validate_redirect_url.validate_url('/test')



import json

from ievv_opensource.utils.ievvbuildstatic import pluginbase
from ievv_opensource.utils.shellcommandmixin import ShellCommandMixin


class Plugin(pluginbase.Plugin, ShellCommandMixin):
    """
    Autosetup jsdoc config and npm script.

    Examples:

        It is really simple to use::

            IEVVTASKS_BUILDSTATIC_APPS = ievvbuildstatic.config.Apps(
                ievvbuildstatic.config.App(
                    appname='demoapp',
                    version='1.0.0',
                    plugins=[
                        ievvbuildstatic.autosetup_jsdoc.Plugin(),
                    ]
                )
            )

        If you need to adjust the config, simply setup
        your own ``build-docs`` script in package.json instead
        of using this plugin.
    """
    name = 'autosetup_jsdoc'

    def install(self):
        self.app.get_installer('npm').queue_install(
            package='jsdoc',
            installtype='dev'
        )

    def get_jsdoc_config_filename(self):
        return 'jsdoc.config.json'

    def get_jsdoc_config_path(self):
        return self.app.get_source_path(self.get_jsdoc_config_filename())

    def make_jsdoc_config_dict(self):
        return {
            "opts": {
                "encoding": "utf8",
                "destination": "./built_docs/",
                "recurse": True
            },
            "source": {
                "includePattern": ".+\\.js(doc|x)?$",
                "excludePattern": "(^|\\/|\\\\)_",
                "include": [
                    "./scripts/javascript/"
                ]
            },
            "tags": {
                "allowUnknownTags": True,
                "dictionaries": ["jsdoc", "closure"]
            },
            "plugins": ["plugins/markdown"],
            "markdown": {
                "parser": "gfm"
            },
            "templates": {
                "cleverLinks": False,
                "monospaceLinks": False
            }
        }

    def create_jsdoc_config(self):
        open(self.get_jsdoc_config_path(), 'w').write(
            json.dumps(
                self.make_jsdoc_config_dict(),
                indent=2,
                sort_keys=True
            ))

    def run(self):
        self.get_logger().command_start('Autosetup jsdoc for {appname}'.format(
            appname=self.app.appname))
        self.create_jsdoc_config()
        self.app.get_installer('npm').add_npm_script(
            'build-docs', 'jsdoc -c {configpath}'.format(
                configpath=self.get_jsdoc_config_filename()))
        self.get_logger().command_success('Autoset jsdoc config succeeded :)')



import os
import shutil

from ievv_opensource.utils.ievvbuildstatic import pluginbase


class Plugin(pluginbase.Plugin):
    # Copy specified files from staticsources/<packagename>/node_modules/foo/bar/<libfile>.js to static/<packagename>/<version>/node_modules/<libfile>.js

    def __init__(self,
                 sourcefiles,
                 destinationfolder=os.path.join('scripts', 'node_modules'),
                 sourcefolder='node_modules',
                 **kwargs):
        super(Plugin, self).__init__(**kwargs)
        self.sourcefiles = sourcefiles
        self.destinationfolder = destinationfolder
        self.sourcefolder = sourcefolder

    def get_sourcefolder_path(self):
        return self.app.get_source_path(self.sourcefolder)

    def get_destinationfolder_path(self):
        return self.app.get_destination_path(self.destinationfolder)

    def get_filepaths(self):
        files = []
        sourcefolder = self.get_sourcefolder_path()
        for file in self.sourcefiles:
            files.append(os.path.join(sourcefolder, file))
        # no idea why regexfilelist wont work..
        # regex_file_list = RegexFileList(include_patterns=files)
        # file_list = regex_file_list.get_files_as_list(sourcefolder, absolute_paths=True)
        # print(file_list)
        return files

    def run(self):
        filepaths = self.get_filepaths()
        destinationfolder = self.get_destinationfolder_path()
        if os.path.exists(destinationfolder):
            self.get_logger().debug('Removing {}'.format(destinationfolder))
            shutil.rmtree(destinationfolder)

        os.makedirs(destinationfolder)

        for sourcepath in filepaths:
            if os.path.exists(sourcepath) and os.path.isfile(sourcepath):
                filename = os.path.basename(sourcepath)
                destinationpath = os.path.join(destinationfolder, filename)
                shutil.copyfile(sourcepath, destinationpath)


    def __str__(self):
        return '{}({})'.format(super(Plugin, self).__str__(), self.sourcefolder)



import os
import re


class RegexFileList(object):
    """
    A list of regexes for matching files.

    Examples:

        Get all javascript files in a directory::

            from ievv_opensource.utils import ievvbuildstatic

            filelist = ievvbuildstatic.utils.RegexFileList(include_patterns=['^.*?\.js$'])
            filelist.get_files_as_list('/path/to/javascript/sources/')

        Exclude some files::

            filelist = ievvbuildstatic.utils.RegexFileList(
                include_patterns=['^.*\.js$'],
                exclude_patterns=['^.*\.spec\.js$']
            )
            filelist.get_files_as_list('/path/to/javascript/sources/')
    """
    def __init__(self, include_patterns=None,
                 exclude_patterns=None):
        self.include_patterns = include_patterns or []
        self.exclude_patterns = exclude_patterns or []

    def matches_include_pattern(self, relative_filepath):
        for pattern in self.include_patterns:
            if re.match(pattern, relative_filepath):
                return True
        return False

    def matches_exclude_pattern(self, relative_filepath):
        for pattern in self.exclude_patterns:
            if re.match(pattern, relative_filepath):
                return True
        return False

    def matches_pattern(self, relative_filepath):
        return (self.matches_include_pattern(relative_filepath) and
                not self.matches_exclude_pattern(relative_filepath))

    def get_files_as_list(self, rootfolder, absolute_paths=False):
        matched_relative_filepaths = []
        for root, dirs, files in os.walk(rootfolder):
            for filename in files:
                filepath = os.path.abspath(os.path.join(root, filename))
                relative_filepath = os.path.relpath(filepath, rootfolder)
                if self.matches_pattern(relative_filepath):
                    if absolute_paths:
                        matched_relative_filepaths.append(filepath)
                    else:
                        matched_relative_filepaths.append(relative_filepath)

        return matched_relative_filepaths

    def prettyformat_patterns(self):
        return 'RegexFileList(include={!r}, exclude={!r})'.format(
            self.include_patterns, self.exclude_patterns)



import os
from django.test import TestCase
from ievv_opensource.utils import virtualenvutils
from ievv_opensource.python2_compatibility import mock


class TestVirtualenvUtils(TestCase):
    def test_is_in_virtualenv_true(self):
        class MockSys:
            real_prefix = '/something'
        with mock.patch('ievv_opensource.utils.virtualenvutils.sys', MockSys):
            self.assertTrue(virtualenvutils.is_in_virtualenv())

    def test_is_in_virtualenv_false(self):
        class MockSys:
            pass
        with mock.patch('ievv_opensource.utils.virtualenvutils.sys', MockSys):
            self.assertFalse(virtualenvutils.is_in_virtualenv())

    def test_get_virtualenv_directory_not_in_virtualenv(self):
        class MockSys:
            pass
        with mock.patch('ievv_opensource.utils.virtualenvutils.sys', MockSys):
            with self.assertRaises(OSError):
                virtualenvutils.get_virtualenv_directory()

    def test_get_virtualenv_directory_is_in_virtualenv(self):
        class MockSys:
            real_prefix = '/something'
            prefix = '/virtual'
        with mock.patch('ievv_opensource.utils.virtualenvutils.sys', MockSys):
            self.assertEqual('/virtual', virtualenvutils.get_virtualenv_directory())

    def test_add_virtualenv_bin_directory_to_path(self):
        class MockSys:
            real_prefix = '/real'
            prefix = '/virtual'
        mockenviron = {'PATH': ''}
        with mock.patch('ievv_opensource.utils.virtualenvutils.sys', MockSys):
            with mock.patch('ievv_opensource.utils.virtualenvutils.os.environ',
                            mockenviron):
                virtualenvutils.add_virtualenv_bin_directory_to_path()
        self.assertEqual('{}/virtual/bin'.format(os.pathsep),
                         mockenviron['PATH'])



from django.core.management.base import BaseCommand

from ievv_opensource.utils import secret_key


class Command(BaseCommand):
    help = 'Generate a Django SECRET_KEY.'

    def add_arguments(self, parser):
        parser.add_argument('--plain', dest='plain',
                            required=False, action='store_true',
                            default=False,
                            help='Do not include SECRET_KEY= in output.')
        parser.add_argument('--heroku', dest='heroku',
                            required=False, action='store_true',
                            default=False,
                            help='Output with heroku config:set ...')

    def handle(self, *args, **options):
        key = secret_key.generate_django_secret_key()
        if options['plain']:
            pass
        elif options['heroku']:
            key = 'heroku config:set DJANGO_SECRET_KEY="{}" --app myapp'.format(key)
        else:
            key = 'SECRET_KEY="{}"'.format(key)
        self.stdout.write(key)



import os
import platform
from ievv_opensource.utils.desktopnotifications import osxnotification
from ievv_opensource.utils.desktopnotifications import linuxnotification
from ievv_opensource.utils.desktopnotifications import mocknotification


if platform.system().lower() == 'darwin':
    _notificationbackend = osxnotification.Notification()
elif platform.system().lower() == 'linux':
    _notificationbackend = linuxnotification.Notification()
else:
    _notificationbackend = mocknotification.Notification()


def show_message(title, message):
    """
    Show a message in the desktop notification system of the OS.

    Examples::

        Show a message::

            desktopnotificationapi.show_message(
                title='Hello world',
                message='This is a test')

    Args:
        title (str): The title of the notification.
        message (str): The notification message.
    """
    enabled = os.environ.get('IEVV_OPENSOURCE_ENABLE_DESKTOP_NOTIFICATIONS', 'true')
    if enabled.lower() == 'true':
        _notificationbackend.show_message(title=title, message=message)


# if __name__ == '__main__':
#     show_message('Hello world', 'Testing "d\'s\'ad')



from datetime import timedelta

from model_mommy.timezone import now
from psycopg2.extras import DateTimeTZRange, DateRange


def generate_datetime_range():
    return DateTimeTZRange(
        lower=now() - timedelta(hours=1),
        upper=now() + timedelta(hours=1)
    )


def generate_date_range():
    return DateRange(
        lower=now().date(),
        upper=(now() + timedelta(days=1)).date()
    )


def add_to_mommy():
    from model_mommy import mommy
    mommy.generators.add(
        'django.contrib.postgres.fields.ranges.DateTimeRangeField',
        generate_datetime_range)
    mommy.generators.add(
        'django.contrib.postgres.fields.ranges.DateRangeField',
        generate_date_range)



import json

from .abstract_npm_installer import AbstractNpmInstaller
from ievv_opensource.utils.shellcommandmixin import ShellCommandError


class NpmInstallerError(Exception):
    pass


class PackageJsonDoesNotExist(NpmInstallerError):
    pass


class YarnInstaller(AbstractNpmInstaller):
    """
    Yarn installer.
    """
    name = 'yarninstall'
    optionprefix = 'yarn'

    def yarn_json_output_handler(self, outputdict):
        ignored_messagetypes = {
            'progressTick',
            'step',
        }
        ignored_messages = {
            'Saved 0 new dependencies.',
            'Saved lockfile.',
        }
        if isinstance(outputdict, dict):
            messagetype = outputdict.get('type')
            data = outputdict.get('data', {})
            if isinstance(data, dict):
                message = data.get('message')
            elif isinstance(data, str):
                message = data
            else:
                message = None
        else:
            message = str(outputdict)
            messagetype = 'unknown'
        if message and messagetype and message not in ignored_messages:
            if messagetype == 'error':
                super(YarnInstaller, self).log_shell_command_stderr(line=message)
            elif messagetype not in ignored_messagetypes:
                super(YarnInstaller, self).log_shell_command_stdout(line=message)

    def log_shell_command_stdout(self, line):
        try:
            outputdict = json.loads(line)
        except ValueError:
            super(YarnInstaller, self).log_shell_command_stdout(line=line)
        else:
            self.yarn_json_output_handler(outputdict=outputdict)

    def _run_yarn(self, args):
        self.run_shell_command('yarn',
                               args=args,
                               _cwd=self.app.get_source_path())

    def log_shell_command_stderr(self, line):
        try:
            outputdict = json.loads(line)
        except ValueError:
            super(YarnInstaller, self).log_shell_command_stdout(line=line)
        else:
            self.yarn_json_output_handler(outputdict=outputdict)

    def install_packages_from_packagejson(self):
        try:
            self._run_yarn(args=['--json'])
        except ShellCommandError:
            self.get_logger().command_error('yarn FAILED!')
            self.add_deferred_warning('Installing packages from package.json failed')
            raise SystemExit()

    def install_npm_package(self, package, properties):
        package_spec = package
        if properties['version']:
            package_spec = '{package}@{version}'.format(
                package=package, version=properties['version'])
        args = ['add', '--json', package_spec]
        if properties['installtype'] is not None:
            args.append('--{}'.format(properties['installtype']))
        try:
            self._run_yarn(args=args)
        except ShellCommandError:
            self.get_logger().command_error(
                'yarn add {package} (properties: {properties!r}) FAILED!'.format(
                    package=package, properties=properties))
            raise SystemExit()

    def uninstall_npm_package(self, package):
        try:
            self._run_yarn(args=['remove', package])
        except ShellCommandError:
            message = 'yarn remove {package} FAILED!'.format(
                    package=package)
            self.get_logger().command_error(message)

    def run_packagejson_script(self, script, args=None):
        args = args or []
        args = ['run', script] + args
        self._run_yarn(args=args)

    def link_package(self, packagename):
        self._run_yarn(args=['link', packagename])

    def unlink_package(self, packagename):
        self._run_yarn(args=['unlink', packagename])

    def install(self):
        self.get_logger().command_start(
            'Installing npm packages for {}'.format(self.app.get_source_path()))
        if self.get_option('clean_node_modules', False):
            self.remove_node_modules_directory()
        self.handle_linked_packages()
        if not self.packagejson_exists():
            self.create_packagejson()
        self.install_packages_from_packagejson()
        self.install_queued_packages()
        self.get_logger().command_success('Install npm packages succeeded :)')
        self.add_deferred_success('Install npm packages succeeded :)')



from django.template import Library, defaulttags

from ievv_opensource.ievv_i18n_url import i18n_url_utils

register = Library()

# TODO: {% url %} compatible template tag!
# @register.simple_tag(takes_context=True)
# def i18n_url(context, viewname, urlconf=None, args=None, kwargs=None, current_app=None, languagecode=None, **kwargs):
#     """Template tag for the ievv_i18n_utils.i18n_url function.

#     You should use this instead of ``{% url %}``, but be warned that this
#     is not a perfect drop in replacement because ``{% url %}`` does some
#     magic with the first argument that we have not re-implemented. I.e.: This
#     template tag is more similar to the ``reverse()`` function than the
#     ``{% url %}`` template tag.

#     Args:
#         viewname: See the docs for ``django.urls.reverse``.
#         urlconf: See the docs for ``django.urls.reverse``. Defaults to None.
#         current_app: See the docs for ``django.urls.reverse``. Defaults to None.
#         kwargs: See the docs for ``django.urls.reverse``. Defaults to no kwargs.

#     Returns:
#         str: An url.
#     """
#     request = context['request']
#     defaulttags.url()
#     return i18n_url_utils.i18n_url(request, *args, **kwargs)


@register.simple_tag(takes_context=True)
def transform_url_to_languagecode(context, *args, **kwargs):
    """Template tag for the ievv_i18n_utils.transform_url_to_languagecode function.

    See :func:`~ievv_opensource.ievv_i18n_url.i18n_url_utils.transform_url_to_languagecode` for
    the available arguments, but do not provide the ``request`` argument - we get that
    from ``context["request"]``.

    Returns:
        str: An url.
    """
    request = context['request']
    return i18n_url_utils.transform_url_to_languagecode(request, *args, **kwargs)



from django.utils import translation

from . import abstract_handler


class DjangoSessionHandler(abstract_handler.AbstractHandler):
    """
    Django session based i18n url handler.

    .. warning::

        Please read the *A warning about session based translations* in the docs before
        deciding to use this handler.
    """

    @classmethod
    def detect_current_languagecode(cls, base_url, request):
        return translation.get_language_from_request(request)

    # @classmethod
    # def strip_languagecode_from_urlpath(cls, base_url, languagecode, path):
    #     return path

    def get_languagecode_from_url(self, url):
        return self.active_languagecode

    def build_absolute_url(self, path, languagecode=None, base_url=None):
        base_url = base_url or self.active_base_url
        return base_url.build_absolute_url(path)

    @classmethod
    def transform_url_to_languagecode(cls, url, languagecode, translate_path=True):
        return url



from collections import OrderedDict

from ievv_opensource.utils.singleton import Singleton


class DuplicateKeyError(Exception):
    """
    Raised when adding a key already in a :class:`.ClassRegistrySingleton`
    """
    def __init__(self, registry, key):
        self.registry = registry
        self.key = key
        message = f'Duplicate key, {key!r}, in {registry.get_pretty_classpath()}.'
        super(DuplicateKeyError, self).__init__(message)


class AbstractRegistryItem:
    """
    Base class for :class:`.ClassRegistrySingleton` items.
    """
    @classmethod
    def get_registry_key(cls):
        raise NotImplementedError()

    @property
    def registry_key(self):
        return self.__class__.get_registry_key()

    @classmethod
    def on_add_to_registry(cls, registry):
        """
        Called automatically when the class is added to a :class:`.ClassRegistrySingleton`.
        """

    @classmethod
    def on_remove_from_registry(cls, registry):
        """
        Called automatically when the class is removed from a :class:`.ClassRegistrySingleton`.
        """


class RegistryItemWrapper:
    """
    Registry item wrapper.

    When you add a :class:`.AbstractRegistryItem` to a :class:`.ClassRegistrySingleton`,
    it is stored as an instance of this class.

    You can use a subclass of this class with your registry singleton by overriding
    :meth:`.ClassRegistrySingleton.get_registry_item_wrapper_class`. This enables
    you to store extra metadata along with your registry items, and provided
    extra helper methods.
    """
    def __init__(self, cls, default_instance_kwargs):
        #: The :class:`.AbstractRegistryItem` class.
        self.cls = cls

        #: The default kwargs for instance created with :meth:`.get_instance`.
        self.default_instance_kwargs = default_instance_kwargs

    def make_instance_kwargs(self, kwargs):
        """
        Used by :meth:`.get_instance` to merge ``kwargs`` with ``default_instance_kwargs``.

        Returns:
            dict: The full kwargs fo the instance.
        """
        full_kwargs = {}
        full_kwargs.update(self.default_instance_kwargs)
        full_kwargs.update(kwargs)
        return full_kwargs

    def get_instance(self, **kwargs):
        """
        Get an instance of the :class:`.AbstractRegistryItem`
        class initialized with the provided ``**kwargs``.

        The provided ``**kwargs`` is merged with the ``default_instance_kwargs``,
        with ``**kwargs`` overriding any keys also in ``default_instance_kwargs``.

        Args:
            **kwargs: Kwargs for the class constructor.

        Returns:
            .AbstractRegistryItem: A class instance.
        """
        return self.cls(**self.make_instance_kwargs(kwargs))


class ClassRegistrySingleton(Singleton):
    """
    Base class for class registry singletons - for having a singleton
    of swappable classes. Useful when creating complex libraries with
    classes that the apps using the libraries should be able to swap out
    with their own classes.

    Example::

        class AbstractMessageGenerator(class_registry_singleton.AbstractRegistryItem):
            def get_message(self):
                raise NotImplementedError()


        class SimpleSadMessageGenerator(AbstractMessageGenerator):
            @classmethod
            def get_registry_key(cls):
                return 'sad'

            def get_message(self):
                return 'A sad message'

        class ComplexSadMessageGenerator(AbstractMessageGenerator):
            @classmethod
            def get_registry_key(cls):
                return 'sad'

            def get_message(self):
                return random.choice([
                    'Most people are smart, but 60% of people think they are smart.',
                    'Humanity will probably die off before we become a multi-planet spiecies.',
                    'We could feed everyone in the world - if we just bothered to share resources.',
                ])

        class SimpleHappyMessageGenerator(AbstractMessageGenerator):
            @classmethod
            def get_registry_key(cls):
                return 'happy'

            def get_message(self):
                return 'A happy message'

        class ComplexHappyMessageGenerator(AbstractMessageGenerator):
            @classmethod
            def get_registry_key(cls):
                return 'happy'

            def get_message(self):
                return random.choice([
                    'Almost every person you will ever meet are good people.',
                    'You will very likely live to see people land on mars.',
                    'Games are good now - just think how good they will be in 10 years!',
                ])

        class MessageGeneratorSingleton(class_registry_singleton.ClassRegistrySingleton):
            ''''
            We never use ClassRegistrySingleton directly - we always create a subclass. This is because
            of the nature of singletons. If you ise ClassRegistrySingleton directly as your singleton,
            everything added to the registry would be in THE SAME singleton.
            ''''


        class DefaultAppConfig(AppConfig):
            def ready(self):
                registry = MessageGeneratorSingleton.get_instance()
                registry.add(SimpleSadMessageGenerator)
                registry.add(SimpleHappyMessageGenerator)


        class SomeCustomAppConfig(AppConfig):
            def ready(self):
                registry = MessageGeneratorSingleton.get_instance()
                registry.add_or_replace(ComplexSadMessageGenerator)
                registry.add_or_replace(ComplexHappyMessageGenerator)


        # Using the singleton in code
        registry = MessageGeneratorSingleton.get_instance()
        print(registry.get_registry_item_instance('sad').get_message())
        print(registry.get_registry_item_instance('happy').get_message())
    """

    def __init__(self):
        super().__init__()
        self._classmap = OrderedDict()

    def get_registry_item_wrapper_class(self):
        """
        Get the registry item wrapper class.

        Defaults to :class:`.RegistryItemWrapper` which should work well for
        most use cases.
        """
        return RegistryItemWrapper

    def get_pretty_classpath(self):
        return '{}.{}'.format(self.__module__, self.__class__.__name__)

    def __getitem__(self, key):
        """
        Get a class (wrapper) stored in the registry by its key.

        Args:
            key (str): The key.

        Raises:
            KeyError: When the ``key`` is not in the registry.

        Returns:
            .RegistryItemWrapper: You can use this to get the class or to get an instance of the class.
        """
        return self._classmap[key]

    def get(self, key, fallback=None):
        """
        Get a class (wrapper) stored in the registry by its key.

        Args:
            key (str): A registry item class key.
            fallback: Fallback value of the ``key`` is not in the registry

        Returns:

        Returns:
            .RegistryItemWrapper: You can use this to get the class or to get an instance of the class.

        """
        if key in self:
            return self[key]
        return fallback

    def __contains__(self, key):
        return key in self._classmap

    def __iter__(self):
        """
        Iterate over all the keys in the registry.
        """
        return iter(self._classmap)

    def items(self):
        """
        Iterate over all the items in the registry yielding (key, RegistryItemWrapper)
        tuples.
        """
        return self._classmap.items()

    def iterwrappers(self):
        """
        Iterate over all the items in the registry yielding RegistryItemWrapper
        objects.
        """
        return self._classmap.values()

    def iterchoices(self):
        """
        Iterate over the the classes in the
        in the registry yielding two-value tuples where both values are the
        :obj:`~.AbstractRegistryItem.get_registry_key()`.

        Useful when rendering in a ChoiceField.

        Returns:
            An iterator that yields ``(<key>, <key>)`` tuples for each
            :class:`.AbstractRegistryItem`
            in the registry. The iterator is sorted by
            :obj:`~.AbstractRegistryItem.get_registry_key()`.
        """
        for cls in sorted(self._classmap.values(), key=lambda wrapper: wrapper.cls.get_registry_key()):
            yield (cls.get_registry_key(),
                   cls.get_registry_key())

    def _set(self, cls, **default_instance_kwargs):
        key = cls.get_registry_key()
        if key in self._classmap:
            self._classmap[key].cls.on_remove_from_registry(registry=self)
        self._classmap[key] = RegistryItemWrapper(
            cls=cls, default_instance_kwargs=default_instance_kwargs)
        self._classmap[key].cls.on_add_to_registry(registry=self)

    def add(self, cls, **default_instance_kwargs):
        """
        Add the provided ``cls`` to the registry.
        Args:
            cls: A :class:`.AbstractRegistryItem` class (NOT AN OBJECT/INSTANCE).
            **default_instance_kwargs: Default instance kwargs.
        Raises:
            .DuplicateKeyError: When a class with the same
            :obj:`~.AbstractRegistryItem.get_registry_key()` is already
            in the registry.
        """
        key = cls.get_registry_key()
        if key in self._classmap:
            raise DuplicateKeyError(
                registry=self,
                key=key)
        self._set(cls, **default_instance_kwargs)

    def add_or_replace(self, cls, **default_instance_kwargs):
        """
        Insert the provided ``cls`` in registry.
        If another ``cls`` is already
        registered with the same ``key``, this will be replaced.

        Args:
            cls: A :class:`.AbstractRegistryItem` class (NOT AN OBJECT/INSTANCE).
            **default_instance_kwargs: Default instance kwargs.
        """
        self._set(cls, **default_instance_kwargs)

    def replace(self, cls, **default_instance_kwargs):
        """
        Replace the class currently in the registry with the same key
        as the provided ``cls.get_registry_key()``.

        Args:
            cls: A :class:`.AbstractRegistryItem` class (NOT AN OBJECT/INSTANCE).
            **default_instance_kwargs: Default instance kwargs.

        Raises:
            KeyError: If the ``cls.get_registry_key()`` is NOT in the registry.
        """
        key = cls.get_registry_key()
        if key not in self._classmap:
            raise KeyError(f'{key!r} is not in the registry.')
        self._set(cls, **default_instance_kwargs)

    def remove(self, key):
        """
        Remove the class provided ``key`` from the registry.

        Args:
            key (str): A
            :obj:`~.AbstractRegistryItem.get_registry_key()`.
        Raises:
            KeyError: When the ``key`` is not in the registry.
        """
        if key not in self:
            raise KeyError
        self._classmap[key].cls.on_remove_from_registry(registry=self)
        del self._classmap[key]

    def remove_if_in_registry(self, key):
        """
        Works just like :meth:`.remove`, but if the ``key`` is not in the registry
        this method just does nothing instead of raising KeyError.
        """
        if key not in self:
            return
        self.remove(key)

    def get_registry_item_instance(self, key, **kwargs):
        """
        Just a shortcut for ``singleton[key].get_instance(**kwargs)``.
        """
        return self[key].get_instance(**kwargs)



from django.conf import settings
from django.utils.module_loading import import_string


_handler_class = None


def get_handler_class():
    """
    Get the configured `ievv_i18n_url` handler class.

    E.g. import the handler class from the class path configured in the ``IEVV_I18N_URL_HANDLER`` setting.

    Returns:
        ievv_opensource.ievv_i18n_url.handlers.abstract_handler.AbstractHandler: A handler class.
    """
    global _handler_class
    if _handler_class is None:
        handler_classpath = getattr(settings, 'IEVV_I18N_URL_HANDLER', None)
        if not handler_classpath:
            raise Exception(
                'No ievv_i18n_url_handler configured. Please set the IEVV_I18N_URL_HANDLER. Refer to the docs for '
                'ievv_i18n_url in ievv_opensource for more info.')
        _handler_class = import_string(handler_classpath)
    return _handler_class


def get_handler():
    return get_handler_class()()



class ValidationErrorUtil(object):
    """
    A wrapper around :class:`django.core.exceptions.ValidationError`
    that provides useful extra functionality.

    Examples:

        Get a serializable dict::

            ValidationErrorUtil(some_validation_error).as_serializable_dict()

        Convert ValidationError to :class:`rest_framework.exceptions.ValidationError`::

            ValidationErrorUtil(some_validation_error).as_drf_validation_error()
    """
    def __init__(self, validation_error):
        self.validation_error = validation_error

    @property
    def is_dict_based(self):
        return hasattr(self.validation_error, 'error_dict')

    @property
    def is_single_message(self):
        return hasattr(self.validation_error, 'message')

    def as_dict(self):
        """
        Get the ValidationError as a dict mapping fieldname to
        a list of ValidationError object.

        Works no matter how the ValidationError was created.
        If the ValidationError was created with ``ValidationError('message')``
        or ``ValidationError(['message1', 'message2'])`` the values will
        end up in the ``__all__`` key of the dict.
        """
        if self.is_dict_based:
            return self.validation_error.error_dict
        return {
            '__all__': self.validation_error.error_list
        }

    def as_list(self):
        """
        Get the ValidationError as a list of ``ValidationError`` objects.

        Works even if the ValidationError was created with a dict as input.
        """
        if self.is_dict_based:
            validation_error_list = []
            for error_list in self.validation_error.error_dict.values():
                validation_error_list.extend(error_list)
            return validation_error_list
        else:
            return self.validation_error.error_list

    def as_serializable_list(self):
        """
        Get the ValidationError as a serializable list (a flat list with all the error messages).
        """
        return [validation_error.message
                for validation_error in self.as_list()]

    def as_serializable_dict(self):
        """
        Get the ValidationError as a serializable dict - a dict mapping
        fieldname to a list of error messages. If the ValidationError was
        not created with a dict as input, the error messages will be
        added to the ``__all__`` key.
        """
        serializable_dict = {}
        for field, validation_error_list in self.as_dict().items():
            serializable_dict[field] = []
            for validation_error in validation_error_list:
                serializable_dict[field].extend(self.__class__(validation_error).as_serializable_list())
        return serializable_dict

    def _as_drf_error_detail(self):
        from rest_framework import exceptions
        return exceptions.ErrorDetail(
            self.validation_error.message or exceptions.ValidationError.default_detail,
            code=self.validation_error.code or exceptions.ValidationError.default_code)

    def as_drf_validation_error(self):
        """
        Convert the ValidationError to a :class:`rest_framework.exceptions.ValidationError`.
        """
        from rest_framework import exceptions
        if self.is_dict_based:
            error_dict = {}
            for field, validation_error_list in self.as_dict().items():
                error_dict[field] = []
                for validation_error in validation_error_list:
                    error_dict[field].append(self.__class__(validation_error)._as_drf_error_detail())
            return exceptions.ValidationError(detail=error_dict)
        elif self.is_single_message:
            return exceptions.ValidationError(self._as_drf_error_detail())
        else:
            error_message_list = []
            for validation_error in self.as_list():
                error_message_list.append(self.__class__(validation_error)._as_drf_error_detail())
            return exceptions.ValidationError(detail=error_message_list)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2019-11-23 11:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ievv_logging', '0002_auto_20191106_1310'),
    ]

    operations = [
        migrations.AddField(
            model_name='ievvloggingeventbase',
            name='error_occured',
            field=models.NullBooleanField(default=None),
        ),
        migrations.AddField(
            model_name='ievvloggingeventitem',
            name='error_occured',
            field=models.NullBooleanField(default=None),
        ),
    ]



import requests
from django.conf import settings

from ievv_opensource.ievv_sms import sms_registry


class Backend(sms_registry.AbstractSmsBackend):
    """
    A pswin (https://pswin.com) backend.

    To use this backend, you should set the following Django settings:

    - ``PSWIN_USERNAME``: The pswin username (USER).
    - ``PSWIN_PASSWORD``: The pswin password (PWD).
    - ``PSWIN_SENDER``: The pswin sender (SND).
    - ``PSWIN_DEFAULT_COUNTRY_CODE``: The country code to use if received
      phone numbers do not start with ``+<code>`` or ``00<code>``.
      E.g.: 47 for norway.

    If you do not set these settings, you must include
    them as kwargs each time you send a message, and this
    makes it hard to switch SMS sending provider.

    The backend_id for this backend is ``pswin``.
    """
    CHARACTER_REPLACE_MAP = {
        '–': '-'
    }

    @classmethod
    def get_backend_id(cls):
        return 'pswin'

    def __init__(self, phone_number, message,
                 pswin_sender=None,
                 pswin_username=None,
                 pswin_password=None,
                 pswin_default_country_code=None,
                 **kwargs):
        self._pswin_sender = pswin_sender
        self._pswin_username = pswin_username
        self._pswin_password = pswin_password
        self._pswin_default_country_code = pswin_default_country_code
        super().__init__(phone_number=phone_number, message=message, **kwargs)

    @property
    def pswin_base_url(self):
        # https://wiki.pswin.com/Gateway%20HTTP%20API.ashx#Submitting_SMS_24
        return 'https://simple.pswin.com'

    @property
    def pswin_username(self):
        return self._pswin_username or settings.PSWIN_USERNAME

    @property
    def pswin_password(self):
        return self._pswin_password or settings.PSWIN_PASSWORD

    @property
    def pswin_sender(self):
        return self._pswin_sender or self.send_as or settings.PSWIN_SENDER

    @property
    def default_country_code(self):
        return self._pswin_default_country_code or settings.PSWIN_DEFAULT_COUNTRY_CODE

    def replace_message_characters(self, message):
        for from_char, to_char in self.CHARACTER_REPLACE_MAP.items():
            message = message.replace(from_char, to_char)
        return message

    def clean_message(self, message):
        message = self.replace_message_characters(message)
        message = message.encode('latin-1', errors='ignore').decode('latin-1')
        return message

    def clean_phone_number(self, phone_number):
        phone_number = self.STRIP_WHITESPACE_PATTERN.sub('', phone_number)
        if phone_number.startswith('00'):
            phone_number = phone_number[2:]
        elif phone_number.startswith('+'):
            phone_number = phone_number[1:]
        else:
            phone_number = '{}{}'.format(self.default_country_code, phone_number)
        return phone_number

    @property
    def pswin_postdata(self):
        return {
            'USER': self.pswin_username,
            'PW': self.pswin_password,
            'RCV': self.cleaned_phone_number,
            'SND': self.pswin_sender,
            'TXT': self.cleaned_message.encode('ISO-8859-1')
        }

    def send(self):
        """
        Send the message using pswin.
        """
        requests.post(self.pswin_base_url, params=self.pswin_postdata)
